%% Calculation of inverter semiconductor power loss: conduction loss, switching loss and total loss

Vbk = VB/(n-1); % Switch blocking voltage
Ron = Ron_per_Vbk*Vbk; % Switch ON resistance [Ohm]

Nsc = fs/fo; % Number of switching cycles in one line cycle
angle = 360/Nsc:360/Nsc:360; % Vector of line-cycle angles [degrees]

ileg = sqrt(2)*Irms*cos((pi/180)*(angle+Phi)); % Vector of leg current [A]

for i=1:3
    d(i,:) = (m/sqrt(3))*cos(angle*pi/180-(i-1)*2*pi/3); % Phase modulating signals; i.e., normalized value of the desired phase voltages
end
dmax = max(d);
dmin = min(d);

switch n
  case 2
   d1(2,:) = 0.5+d(1,:)-(dmax+dmin)/2; % Duty ratio of connection of phase 1 to dc-link point 2
   d1(1,:) = 1-d1(2,:); % Duty ratio of connection of phase 1 to dc-link point 1
   Econd_sc = Ron*ileg.*ileg/fs; % Conduction energy loss in each switching cycle [J]
   Nlt = (d1(1,:)>0)+(d1(2,:)>0)-1; % Number of transitions between levels within half a switching cycle
  case 3
   d1(1,:)=dmax-d(1,:); % Duty ratio of connection of phase 1 to dc-link point 1
   d1(3,:)=d(1,:)-dmin; % Duty ratio of connection of phase 1 to dc-link point 3
   d1(2,:)=1-d1(1,:)-d1(3,:); % Duty ratio of connection of phase 1 to dc-link point 2
   Econd_sc = (2*d1(1,:)+d1(2,:)+2*d1(3,:)).*ileg.*ileg*Ron/fs; % Conduction energy loss in each switching cycle  [J]
   Nlt = (d1(1,:)>0)+(d1(2,:)>0)+(d1(3,:)>0)-1; % Number of transitions between levels within half a switching cycle
  case 4
   d1(1,:)=dmax-d(1,:); % Duty ratio of connection of phase 1 to dc-link point 1
   d1(4,:)=d(1,:)-dmin; % Duty ratio of connection of phase 1 to dc-link point 4
   d1(2,:)=(1-d1(1,:)-d1(4,:))/2; % Duty ratio of connection of phase 1 to dc-link point 2
   d1(3,:)=d1(2,:); % Duty ratio of connection of phase 1 to dc-link point 3
   Econd_sc = (3*d1(1,:)+1.4*d1(2,:)+1.4*d1(3,:)+3*d1(4,:)).*ileg.*ileg*Ron/fs; % Conduction energy loss in each switching cycle  [J]
   Nlt = (d1(1,:)>0)+(d1(2,:)>0)+(d1(3,:)>0)+(d1(4,:)>0)-1; % Number of transitions between levels within half a switching cycle
  otherwise
            disp('Selected number of levels not supported')
end

Pcond = 3*sum(Econd_sc)*fo; % Average three-phase inverter conduction power loss [W]

Eoff = Vbk*Vbk*abs(ileg)/(2*sv)+Vbk*ileg.*ileg/(2*si); % Hard switching turn-off energy loss [J]
Eon = Eoff*(1+DeltaEONloss*Vbk/100); % Hard switching turn-on energy loss [J]
Esw_sc= (Eoff+Eon).*Nlt; % Switching energy loss in each switching cycle [J]
Psw = 3*sum(Esw_sc)*fo; % Average three-phase inverter switching power loss [W]

Ptot = Pcond + Psw; % Total semiconductor average power loss [W]