%% Calculation of inverter line-to-line voltage: as a function of time, spectrum, and THD

Vel = VB/(n-1); % Elementary voltage
N = 2^ceil(log2(N_Ts_min*fs/fo)); % Number of points in time per fundamental cycle (it is forced to be a power of 2)
To = 1/fo; % Fundamental period [s]
Ts = 1/fs;
dt = To/N; % time interval between points
time = dt:dt:To; % Vector of points in time
fr = fo:fo:fo*N/2; % Vector of frequencies
car = 1-abs(Ts/2-rem(time,Ts))/(Ts/2); % Simmetrical triangular carrier waveform

for i=1:3
    d(i,:) = (m/sqrt(3))*cos(2*pi*fo*time-(i-1)*2*pi/3); % Phase modulating signals; i.e., normalized value of the desired phase voltages
end
dmax = max(d);
dmin = min(d);

switch n
  case 2
   d1(2,:) = 0.5+d(1,:)-(dmax+dmin)/2; % Duty ratio of connection of phase 1 to dc-link point 2
   d1(1,:) = 1-d1(2,:); % Duty ratio of connection of phase 1 to dc-link point 1
   d2(2,:) = 0.5+d(2,:)-(dmax+dmin)/2; % Duty ratio of connection of phase 2 to dc-link point 2
   d2(1,:) = 1-d2(2,:); % Duty ratio of connection of phase 2 to dc-link point 1
   V1 = Vel*(car>d1(1,:));
   V2 = Vel*(car>d2(1,:));
  case 3
   d1(1,:)=dmax-d(1,:); % Duty ratio of connection of phase 1 to dc-link point 1
   d1(3,:)=d(1,:)-dmin; % Duty ratio of connection of phase 1 to dc-link point 3
   d1(2,:)=1-d1(1,:)-d1(3,:); % Duty ratio of connection of phase 1 to dc-link point 2
   d2(1,:)=dmax-d(2,:); % Duty ratio of connection of phase 2 to dc-link point 1
   d2(3,:)=d(2,:)-dmin; % Duty ratio of connection of phase 2 to dc-link point 3
   d2(2,:)=1-d2(1,:)-d2(3,:); % Duty ratio of connection of phase 2 to dc-link point 2
   V1 = Vel*((car>d1(1,:))+(car>(d1(1,:)+d1(2,:))));
   V2 = Vel*((car>d2(1,:))+(car>(d2(1,:)+d2(2,:))));
  case 4
   d1(1,:)=dmax-d(1,:); % Duty ratio of connection of phase 1 to dc-link point 1
   d1(4,:)=d(1,:)-dmin; % Duty ratio of connection of phase 1 to dc-link point 4
   d1(2,:)=(1-d1(1,:)-d1(4,:))/2; % Duty ratio of connection of phase 1 to dc-link point 2
   d1(3,:)=d1(2,:); % Duty ratio of connection of phase 1 to dc-link point 3
   d2(1,:)=dmax-d(2,:); % Duty ratio of connection of phase 2 to dc-link point 1
   d2(4,:)=d(2,:)-dmin; % Duty ratio of connection of phase 2 to dc-link point 4
   d2(2,:)=(1-d2(1,:)-d2(4,:))/2; % Duty ratio of connection of phase 2 to dc-link point 2
   d2(3,:)=d2(2,:); % Duty ratio of connection of phase 2 to dc-link point 3
   V1 = Vel*((car>d1(1,:))+(car>(d1(1,:)+d1(2,:)))+(car>(d1(1,:)+d1(2,:)+d1(3,:))));
   V2 = Vel*((car>d2(1,:))+(car>(d2(1,:)+d2(2,:)))+(car>(d2(1,:)+d2(2,:)+d2(3,:))));
  otherwise
            disp('Selected number of levels not supported')
end
Vll = V1-V2;

% FFT and THD computation

auxv = fft(Vll);
for k=1:N/2,
    av(k)=2*real(auxv(k+1))/N;
    bv(k)=-2*imag(auxv(k+1))/N;
end
vharm2=av.*av+bv.*bv;
vharm=sqrt(vharm2); % Peak value of the harmonic
THDv=sqrt(sum(vharm2(2:N/2))/vharm2(1))*100; %[%]

figure;
set(groot,'defaultLineLineWidth',1)
set(groot,'defaultAxesFontSize',12)
plot(time*1e3,Vll);
axis([0 2 -1000 1000]);
%set(groot,'defaultAxesXTick',[0,0.5,1,1.5,2])
figure;
plot(fr/1e3,vharm);
axis([0 500 0 VB]);